<!DOCTYPE html>
<html lang="en">
<head>
    <title>FrImCla image class predict</title>
    <meta http-equiv="content-language" content="en">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
    <meta name="description" content="Web application of FrImCla framework to classify images">
    <meta name="author" content="Manuel García">
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.12.4/jquery.min.js"></script>
    <script src="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/js/bootstrap.min.js"></script>
    <link rel="stylesheet" href="https://maxcdn.bootstrapcdn.com/bootstrap/3.3.7/css/bootstrap.min.css">
    <link href="../static/signup.css" rel="stylesheet">
</head>
 {% block body %}{% endblock %}
<footer class="footer">
    <div class="panel panel-default">
        <div class="panel-footer">&copy; Universidad de La Rioja 2018</div>
    </div>
</footer>
</body>
</html>
